import pandas as pd


class IMPSummaryDashboard:

    import pandas as pd

    def imp_summary_dashboard(self
                              , username: str
                              , warehouse: str
                              , database: str
                              , role: str
                              , date_min: str = '1900-01-01'
                              , date_max: str = '9999-12-31'
                              , column_filters=None):
        """
        Generate a SQL query for a dashboard summary report with optional column filters.

        Args:
            date_min (str): The minimum activity end date to include in the report. Default is '1900-01-01'.
            date_max (str): The maximum activity end date to include in the report. Default is '9999-12-31'.
            column_filters (list): A list of column names to include in the query SELECT and GROUP BY clauses. Default
            is None.

        Returns:
            DataFrame: .

        Raises:
            TypeError: If date_min, date_max, or column_filters is not a string or list, respectively.
            :param column_filters:
            :param date_max:
            :param date_min:
            :param role:
            :param database:
            :param warehouse:
            :param username:
        """

        from NikeCA import Snowflake

        if column_filters is None:
            column_filters = []
        import configparser

        config = configparser.ConfigParser()
        config.read('pip.ini')

        date_begin = config['imp_summary'].get('begin_date')
        date_end = config['imp_summary'].get('date_end')
        date = config['imp_summary'].get('date')
        table = config['imp_summary'].get('table')
        column = config['imp_summary'].get('column')
        column_filter = config['imp_summary'].get('column_filter')
        column_flag = config['imp_summary'].get('column_flag')
        column_flag_answer = config['imp_summary'].get('column_flag_answer')
        aggs = config['imp_summary'].get('aggs')

        # Ensure column_filters is a list
        if not isinstance(column_filters, list):
            raise TypeError("column_filters must be a list")

        # Join column filters into comma-separated string, if any
        if len(column_filters) > 0:
            column_filters_str = ', '.join(column_filters)
            group_column_filters_str = f'''GROUP BY 
                {column_filters_str}'''
            column_filters_str += ', '
        else:
            column_filters_str = column + ', '
            group_column_filters_str = f'GROUP BY {column}'

        query_dashboard = f"""
        SELECT
            {column_filters_str}
            MIN({date_begin}) AS MIN_{date_begin}
            , MAX({date_end}) AS MAX_{date_end}
            , {aggs}
        FROM 
            {table}
        WHERE
            {date} BETWEEN '{date_min}' AND '{date_max}'
            AND {column} = '{column_filter}'
            AND {column_flag} = '{column_flag_answer}'
        {group_column_filters_str} 
        UNION
    
        SELECT
            {column_filters_str}
            MIN({date_begin}) AS MIN_{date_begin}
            , MAX({date_end}) AS MAX_{date_end}
            , {aggs}
        FROM 
            {table}
        WHERE
            {date} BETWEEN '{date_min}' AND '{date_max}'
            AND {column} <> '{column_filter}'
            AND {column_flag} = '{column_flag_answer}'
        {group_column_filters_str}
        ;
        """
        print(query_dashboard)

        return Snowflake(username=username, warehouse=warehouse, database=database,
                         role=role).snowflake_pull(query=query_dashboard)





class BuildSearchQuery:

    def build_search_query(self, inp_db=None, schema=None, table=None, column_name=None, like_flag=False,
                           col_and_or='AND'):
        """
        Constructs a SQL query for searching tables and columns in a database based on specified search criteria.

        Parameters:
        -----------
        inp_db: str, optional
            The database name to search in. If not specified, searches all databases.
        schema: str, optional
            The schema name to search in. If not specified, searches all schemas.
        table: str, optional
            The table name to search for. If not specified, searches all tables.
        column_name: str or list of str, optional
            The column name(s) to search for. If not specified, searches all columns.
            If a list is provided, searches for any columns that match any of the names in the list.
        like_flag: bool, optional
            If True, uses a SQL LIKE statement to search for columns that contain the specified column name(s).
            If False, searches for exact matches to the specified column name(s).
            If not specified, defaults to False.
        col_and_or: str, optional
            If specified and column_name is a list, determines whether to search for columns that match all or any of
            the names in the list. Must be one of the following values: 'AND', 'and', 'OR', 'or'.
            If not specified, defaults to 'AND'.

        Returns:
        --------
        str
            The constructed SQL query string.
        """

        # ie. always TRUE --> allows us to search for tables/cols/etc. even without knowing the db
        where_stmt = "WHERE 1=1 "
        where_stmt = where_stmt + f"AND table_catalog = '{inp_db}' " if inp_db else where_stmt
        where_stmt = where_stmt + f"AND table_schema = '{schema}' " if schema else where_stmt
        where_stmt = where_stmt + f"AND table_name = '{table}' " if table else where_stmt

        # add column(s) search criteria -->
        # if like_flag = false then column name equals
        # if column_name is a list add an AND statement for each search value in the list
        if column_name is not None:
            if type(column_name) == str:
                where_stmt = where_stmt + f"AND column_name like '{column_name}' " \
                    if like_flag else where_stmt + f"AND column_name = '{column_name}' "

            # OR statement where value matches multiple
            elif (type(column_name) == list) & (not like_flag):
                where_stmt = where_stmt + f"""AND column_name in ({' ,'.join(f"'{str(x)}'" for x in column_name)})"""

            # --> user input list of search criteria
            elif type(column_name) == list:
                for idx, x in enumerate(column_name):

                    # col contains all column_name search criteria
                    if col_and_or.lower() == 'and':
                        where_stmt = where_stmt + f"AND column_name like '{x}' " \
                            if like_flag else where_stmt + f"AND column_name = '{x}' "

                    # col contains any of the column_name search criteria
                    elif col_and_or.lower() == 'or':
                        where_stmt = where_stmt + f"AND (column_name like '{x}' " \
                            if idx == 0 else where_stmt + f"OR column_name like '{x}' "

                    # non-matching input value
                    else:
                        raise ValueError('col_and_or input must match: AND/And/and, OR/Or/or')
                where_stmt = where_stmt + ')' if (type(column_name) == list) & (col_and_or == 'or') else where_stmt

            # --> invalid format
            else:
                raise ValueError(f'ERROR: column_name={column_name} does not match required input of list/string')

        # final search-schema query
        query = f'''        
        SELECT 
            DISTINCT
            TABLE_CATALOG
            ,TABLE_SCHEMA
            ,TABLE_NAME
            ,COLUMN_NAME
            ,IS_NULLABLE
            ,DATA_TYPE
        FROM 
            INFORMATION_SCHEMA.COLUMNS
        {where_stmt}
        ORDER BY 
            TABLE_CATALOG
            , TABLE_SCHEMA
            , TABLE_NAME
            , COLUMN_NAME
        '''
        return query






import pandas as pd

from _SnowflakeData import SnowflakeData
from _BuildSearchQuery import BuildSearchQuery
from _SnowflakeDependencies import SnowflakeDependencies
from _IMPSummaryDashboard import IMPSummaryDashboard


class Snowflake(SnowflakeData, IMPSummaryDashboard, SnowflakeDependencies, BuildSearchQuery):
    """
        A class for interacting with Snowflake databases and executing queries.
        Inherits from _SnowflakeData.SnowflakeData.
        """

    # Constructor
    def __init__(self, username: str, warehouse: str, role: str, database: str = None, schema: str = None,
                 table: str = None, column_name: str = None, col_and_or: str = 'AND', get_ex_val: bool = None,
                 like_flag: bool = False, sample_table: bool = False, sample_val: bool = False,
                 table_sample: dict = None, dtypes_conv=None, query='', separate_dataframes: bool = False,
                 tables: list | str | None = None, save_path: str | None = None, date_min: str | None = None,
                 date_max: str | None = None, column_filters: list | None = None, polars: bool = False):
        """
        Parameters:

            username (str): The Snowflake account username.
            warehouse (str): The Snowflake warehouse to use.
            role (str): The Snowflake role to use.
            database (str, optional): The Snowflake database to use (default is None).
            schema (str, optional): The Snowflake schema to use (default is None).
            table (str, optional): The Snowflake table to use (default is None).
            column_name (str, optional): The name of the column to search (default is None).
            col_and_or (str, optional): The AND/OR operator to use between search criteria (default is None).
            get_ex_val (bool, optional): Whether to return exact matches only (default is None).
            like_flag (bool, optional): Whether to use the LIKE operator for search criteria (default is None).
        """

        self.__username = username
        self.__warehouse = warehouse
        self.__role = role
        self.__database = database
        self.__schema = schema
        self.__table = table
        self.__column_name = column_name
        self.__col_and_or = col_and_or
        self.__get_ex_val = get_ex_val
        self.__like_flag = like_flag
        self.__sample_table = sample_table
        self.__sample_val = sample_val
        self.__table_sample = table_sample
        self.__dtypes_conv = dtypes_conv
        self.__query = query
        self.__separate_dataframes = separate_dataframes
        self.__tables = tables
        self.__save_path = save_path
        self.__date_min = date_min
        self.__date_max = date_max
        self.__column_filters = column_filters
        self.__polars = polars

    # Getter and Setter Methods for Instance Variables
    @property
    def username(self):
        return self.__username

    @username.setter
    def username(self, value):
        self.__username = value

    @property
    def warehouse(self):
        return self.__warehouse

    @warehouse.setter
    def warehouse(self, value):
        self.__warehouse = value

    @property
    def role(self):
        return self.__role

    @role.setter
    def role(self, value):
        self.__role = value

    @property
    def database(self):
        return self.__database

    @database.setter
    def database(self, value):
        self.__database = value

    @property
    def schema(self):
        return self.__schema

    @schema.setter
    def schema(self, value):
        self.__schema = value

    @property
    def table(self):
        return self.__table

    @table.setter
    def table(self, value):
        self.__table = value

    @property
    def column_name(self):
        return self.__column_name

    @column_name.setter
    def column_name(self, value):
        self.__column_name = value

    @property
    def col_and_or(self):
        return self.__col_and_or

    @col_and_or.setter
    def col_and_or(self, value):
        self.__col_and_or = value

    @property
    def get_ex_val(self):
        return self.get_ex_val

    @get_ex_val.setter
    def get_ex_val(self, value):
        self.__get_ex_val = value

    @property
    def like_flag(self):
        return self.__like_flag

    @like_flag.setter
    def like_flag(self, value):
        self.__like_flag = value

    @property
    def sample_table(self):
        return self.__sample_table

    @sample_table.setter
    def sample_table(self, value):
        self.__sample_table = value

    @property
    def sample_val(self):
        return self.__sample_val

    @sample_val.setter
    def sample_val(self, value):
        self.__sample_val = value

    @property
    def table_sample(self):
        return self.__table_sample

    @table_sample.setter
    def table_sample(self, value):
        self.__table_sample = value

    @property
    def dtypes_conv(self):
        return self.__dtypes_conv

    @dtypes_conv.setter
    def dtypes_conv(self, value):
        self.__dtypes_conv = value

    @property
    def query(self):
        return self.__query

    @query.setter
    def query(self, value):
        self.__query = value

    @property
    def separate_dataframes(self):
        return self.__separate_dataframes

    @separate_dataframes.setter
    def separate_dataframes(self, value):
        self.__separate_dataframes = value

    @property
    def tables(self):
        return self.__tables

    @tables.setter
    def tables(self, value):
        self.__table = value

    @property
    def save_path(self):
        return self.__save_path

    @save_path.setter
    def save_path(self, value):
        self.__save_path = value

    @property
    def date_min(self):
        return self.__date_min

    @date_min.setter
    def date_min(self, value):
        self.__date_min = value

    @property
    def date_max(self):
        return self.__date_max

    @date_max.setter
    def date_max(self, value):
        self.__date_max = value

    @property
    def column_filters(self):
        return self.__column_filters

    @column_filters.setter
    def column_filters(self, value):
        self.__column_filters = value

    @property
    def polars(self):
        return self.__polars

    @polars.setter
    def polars(self, value):
        self.__polars = value

    def build_search_query(self, inp_db: str = None, schema: str = None, table: str = None,
                           column_name=None, like_flag: bool = False, col_and_or: str = 'AND'):
        """
        Builds and returns a search query based on the specified parameters and instance variables.

        Parameters:
            inp_db (str or None, optional): The database to use (default is None).
            schema (str or None, optional): The schema to use (default is None).
            table (str or None, optional): The table to search (default is None).
            column_name (str or list or None, optional): The name of the column to search (default is None).
            like_flag (bool, optional): Whether to use the LIKE operator for search criteria (default is False).
            col_and_or (str, optional): The AND/OR operator to use between search criteria (default is AND).
        """

        if schema is None:
            schema = self.__schema
        if table is None:
            table = self.__table
        if column_name is None:
            column_name = self.__column_name
        if (not like_flag) and self.__like_flag:
            like_flag = self.__like_flag
        if col_and_or.lower() == 'and' and self.__col_and_or:
            col_and_or = self.__col_and_or

        return BuildSearchQuery.build_search_query(self, inp_db=inp_db, schema=schema, table=table,
                                                   column_name=column_name, like_flag=like_flag,
                                                   col_and_or=col_and_or)

    def snowflake_pull(self,
                       query: str | dict,
                       username: str = None,
                       warehouse: str = None,
                       database: str = None,
                       role: str = None,
                       schema: str = None,
                       table: str = None,
                       sample_table: bool = False,
                       sample_val: bool = False,
                       table_sample: dict = None,
                       dtypes_conv=None,
                       separate_dataframes: bool = True,
                       polars: bool = False):

        """
        Executes a query in Snowflake and returns the results as a Pandas DataFrame.

        Parameters:
            query (str): The SQL query to execute.
            username (str, optional): The Snowflake username to use. If not provided, uses the one set in the class
                                      constructor.
            warehouse (str, optional): The Snowflake warehouse to use. If not provided, uses the one set in the class
                                       constructor.
            database (str, optional): The Snowflake database to use. If not provided, uses the one set in the class
                                      constructor.
            role (str, optional): The Snowflake role to use. If not provided, uses the one set in the class constructor.
            sample_table (bool, optional): Whether to return only a sample of the result table. Defaults to False.
            sample_val (bool, optional): Whether to return only a sample of the result values. Defaults to False.
            table_sample (dict, optional): Dictionary containing the number of rows to return per table in the query.
                                           If not provided, returns all rows. Defaults to None.
            dtypes_conv (dict, optional): Dictionary containing the column names and their desired data types. If not
                                          provided, uses the default data types.

        Returns:
            pandas.DataFrame: The results of the query as a Pandas DataFrame.
            :param polars:
            :param separate_dataframes:
            :param dtypes_conv:
            :param sample_val:
            :param table_sample:
            :param sample_table:
            :param query:
            :param username:
            :param warehouse:
            :param database:
            :param role:
            :param table:
            :param schema:
        """

        # Set default values for the parameters if they are not provided
        if username is None:
            username = self.__username
        if warehouse is None:
            warehouse = self.__warehouse
        if database is None:
            database = self.__database
        if role is None:
            role = self.__role
        if schema is None:
            schema = self.__schema
        if table is None:
            table = self.__table
        if not sample_table:
            sample_table = self.__sample_table
        if not sample_val:
            sample_val = self.__sample_val
        if table_sample is None:
            table_sample = self.__table_sample
        if dtypes_conv is None:
            dtypes_conv = self.__dtypes_conv
        if not separate_dataframes:
            separate_dataframes = self.__separate_dataframes
        if not polars:
            polars = self.__polars

        # Call the snowflake_pull method from the _SnowflakeData module using the provided or default parameters
        return SnowflakeData.snowflake_pull(self, query, un=username, wh=warehouse, db=database,
                                                           role=role, schema=schema, table=table,
                                                           sample_table=sample_table, sample_val=sample_val,
                                                           table_sample=table_sample, dtypes_conv=dtypes_conv,
                                                           separate_dataframes=separate_dataframes, polars=polars)

    def search_schema(self,
                      username: str = None,
                      warehouse: str = None,
                      database: str = None,
                      role: str = None,
                      sample_table: bool = False,
                      sample_val: bool = False,
                      table_sample: dict = None,
                      dtypes_conv=None,
                      schema: str = None,
                      table: str = None,
                      column_name: str = None,
                      col_and_or='and',
                      get_ex_val: bool = False,
                      like_flag: bool = False) -> pd.DataFrame:

        """
        search snowflake structure for specific schema/table/column;
        (optional) specify to return example values from each table-column

        :param username: Any = None
        :param warehouse: Any = None
        :param database: Any = None
        :param role: Any = None
        :param sample_table: bool = False
        :param sample_val: bool = False
        :param table_sample: dict | None = None
        :param dtypes_conv: Any = None
        :param schema: Any = None
        :param table: Any = None
        :param column_name: Any = None
        :param col_and_or: str = 'and'
        :param get_ex_val: bool = False
        :param like_flag: bool = False
        :return: DataFrame
        """

        if username is None:
            username = self.__username
        if warehouse is None:
            warehouse = self.__warehouse
        if database is None:
            database = self.__database
        if role is None:
            role = self.__role
        if not sample_table and self.__sample_table:
            sample_table = self.__sample_table
        if not sample_val and self.__sample_val:
            sample_val = self.__sample_val
        if table_sample is None and self.__table_sample:
            table_sample = self.__table_sample
        if dtypes_conv is None and self.__dtypes_conv:
            dtypes_conv = self.__dtypes_conv
        if schema is None and self.__schema:
            schema = self.__schema
        if table is None and self.__table:
            table = self.__table
        if column_name is None and self.__column_name:
            column_name = self.__column_name
        if col_and_or == 'and' and self.__col_and_or:
            col_and_or = self.__col_and_or
        if not get_ex_val and self.__get_ex_val:
            get_ex_val = self.__get_ex_val
        if not like_flag and self.__like_flag:
            like_flag = self.__like_flag

        return _SnowflakeData.SnowflakeData.search_schema(self, un=username, wh=warehouse, db=database, role=role,
                                                          sample_table=sample_table, sample_val=sample_val,
                                                          table_sample=table_sample, dtypes_conv=dtypes_conv,
                                                          schema=schema, table=table, column_name=column_name,
                                                          col_and_or=col_and_or, get_ex_val=get_ex_val,
                                                          like_flag=like_flag)

    def optimize_tbl_mem(self,
                         username: str | None = None,
                         warehouse: str | None = None,
                         role: str | None = None,
                         database: str | None = None,
                         schema: str | None = None,
                         table_name: str | None = None,
                         pull_all_cols: bool = True,
                         run_debugging: bool = False,
                         query: any = None):

        """
        build a dictionary containing keys that reference column:datatype conversion
        (w/ the purpose of optimizing memory after pulling data)

        :param username: str or None = None
        :param warehouse: str or None = None
        :param database: str or None = None
        :param role: str or None = None
        :param schema: str or None = None
        :param table_name: str or None = None
        :param pull_all_cols: bool = True
        :param run_debugging: bool = False
        :param query: str or None = None
        :return: Dictionary
        """

        if username is None:
            username = self.__username
        if warehouse is None:
            warehouse = self.__warehouse
        if role is None:
            role = self.__role
        if database is None:
            database = self.__database

        return _SnowflakeData.SnowflakeData.optimize_tbl_mem(self, username=username, warehouse=warehouse, role=role,
                                                             database=database, schema=schema, table_name=table_name,
                                                             pull_all_cols=pull_all_cols, run_debugging=run_debugging,
                                                             query=query)

    def snowflake_dependencies(self,
                               tables: str | list | None = None,
                               username: str | None = None,
                               warehouse: str | None = None,
                               role: str | None = None,
                               database: str | None = None,
                               schema: str | list | None = None,
                               save_path: str = None):

        """

        :param save_path:
        :param tables:
        :param username:
        :param warehouse:
        :param role:
        :param database:
        :param schema:
        :return:
        """

        if tables is None:
            tables = self.__tables
        if username is None:
            username = self.__username
        if warehouse is None:
            warehouse = self.__warehouse
        if role is None:
            role = self.__role
        if database is None:
            database = self.__database
        if schema is None:
            schema = self.__schema
        if save_path is None:
            save_path = self.__save_path

        return _SnowflakeData.SnowflakeData.snowflake_dependencies(self, tables=tables, username=username,
                                                                   warehouse=warehouse, role=role, database=database,
                                                                   schema=schema, save_path=save_path)

    def imp_summary_dashboard(self
                              , username: str | None = None
                              , warehouse: str | None = None
                              , database: str | None = None
                              , role: str | None = None
                              , date_min: str = '1900-01-01'
                              , date_max: str = '9999-12-31'
                              , column_filters=None) -> pd.DataFrame:

        if username is None:
            username = self.__username
        if warehouse is None:
            warehouse = self.__warehouse
        if database is None:
            database = self.__database
        if role is None:
            role = self.__role
        if date_min is None:
            date_min = self.__date_min
        if date_max is None:
            date_max = self.__date_max
        if column_filters is None:
            column_filters = self.__column_filters

        return IMPSummaryDashboard.imp_summary_dashboard(self, username=username, warehouse=warehouse, database=database
                                                         , role=role, date_min=date_min, date_max=date_max,
                                                         column_filters=column_filters)








class SnowflakeDependencies:
    import pandas

    def snowflake_dependencies(self,
                               tables: str | list,
                               username: str,
                               warehouse: str,
                               role: str,
                               database: str | None = None,
                               schema: str | list | None = None,
                               save_path: str | None = None):

        """
        Searches the snowflake database and finds instances where the table is referenced and where the reference is not
            in the actual creation of the table itself

        There are some tables in the databases that do not have any results for the get_ddl(). I have set it to skip
            these tables

        :param save_path:
        :param tables: This is a list or string to check for in the database
        :param username: username for Snowflake
        :param warehouse: warehouse for Snowflake
        :param role: role for Snowflake
        :param database: database to search in must provide
        :param schema: filling this in can really help to speed up the query
        :return: pandas Dataframe
        """

        # snowflake connection packages:
        import pandas as pd
        import polars as pl
        import snowflake.connector
        import time
        import json
        import os

        if type(tables) == str:
            tables = [tables]

        print('opening snowflake connection...')

        cnn = snowflake.connector.connect(
            user=username,
            account='nike',
            authenticator='externalbrowser',
            role=role,
            warehouse=warehouse,
        )
        cs = cnn.cursor()
        process_complete = 0
        process_pass = 0
        counter = 0

        # fetch schema and table names
        query = 'SELECT * FROM ' + database + '.INFORMATION_SCHEMA.TABLES'
        if schema is not None:
            if type(schema) == str:
                schema = [schema]
            query = query + " WHERE TABLE_SCHEMA IN ('" + schema[0] + "'"
            if len(schema) > 1:
                for i in schema[1:]:
                    query = query + ", '" + i + "'"
            query = query + ')'

        cs.execute(query)
        df_tables = cs.fetch_pandas_all()

        query_ddl = {}

        for k, i in df_tables.iterrows():
            if i['TABLE_TYPE'] == 'VIEW':
                query_ddl[i['TABLE_CATALOG'] + '.' + i['TABLE_SCHEMA'] + '.' + i['TABLE_NAME']] = \
                    "SELECT GET_DDL('" + i['TABLE_TYPE'] + "', '" + i['TABLE_CATALOG'] + '.' + i[
                        'TABLE_SCHEMA'] + '.' \
                    + i['TABLE_NAME'] + "')"
            elif i['TABLE_TYPE'] == 'BASE TABLE':
                query_ddl[i['TABLE_CATALOG'] + '.' + i['TABLE_SCHEMA'] + '.' + i['TABLE_NAME']] = \
                    "SELECT GET_DDL('TABLE', '" + i['TABLE_CATALOG'] + '.' + i['TABLE_SCHEMA'] + '.' + \
                    i['TABLE_NAME'] + "')"

        df = pd.DataFrame([query_ddl]).T

        df_index = df.index

        df_return = pd.DataFrame(index=df_index)
        df_return['sfqid'] = ''

        queries = len(df)
        print('Pulling ' + str(queries) + ' queries')

        query_list = []
        db_list = []
        complete = []

        for item in range(queries):
            query_list.append(item)
            db_list.append(item)
            complete.append(0)

        for k, v in df.iterrows():
            sql = v[0]
            cs.execute_async(sql)
            query_list[counter] = cs.sfqid
            df_return['sfqid'][k] = cs.sfqid
            counter += 1
        counter = 1
        if save_path is not None:
            if os.path.isfile(save_path):
                with open(save_path) as f:
                    d = json.load(f)
            else:
                d = {}
        else:
            d = {}
        while process_complete == 0:
            item = -1
            process_pass += 1
            if sum(complete) == queries or process_pass == 1000:
                process_complete = 1
            for result in query_list:
                item += 1
                if complete[item] == 0:
                    print('Running ' + df_return[df_return['sfqid'] == result].index[0])
                    try:
                        status = cnn.get_query_status_throw_if_error(result)

                    except snowflake.connector.errors.ProgrammingError:
                        print(f"""Could not retrieve:
                            {df_return[df_return['sfqid'] == result].index[0]}
                            
                            because of snowflake.connector.errors.ProgrammingError""")
                        complete[item] = 1
                        continue

                    except TypeError:
                        print(f"""Could not retrieve:
                            {df_return[df_return['sfqid'] == result].index[0]}
                        
                        because TypeError: NoneType""")
                        complete[item] = 1
                        continue

                    print('the status for ' + df_return[df_return['sfqid'] == result].index[0] + ' is ' +
                          str(status) + ' ' + str(counter))
                    if str(status) == 'QueryStatus.SUCCESS':
                        complete[item] = 1
                        cs.get_results_from_sfqid(result)
                        data = cs.fetch_pandas_all()
                        for table in tables:
                            try:
                                if table.upper() + ' ' in data.iloc[0, 0][data.iloc[0, 0].upper().index(
                                        table.upper()):data.iloc[0, 0].upper().index(table.upper()) + len(table) + 1]\
                                        and table.upper() not in data.iloc[0, 0][:data.iloc[0, 0].index('(')]:
                                    d[table] = {}
                                    d[table][df_return[df_return['sfqid'] == result].index[0]] = data.to_dict('index')
                            except ValueError:
                                continue
                    else:
                        time.sleep(.1)
                counter += 1
        cnn.close()

        if save_path is not None:
            with open(save_path, "w+") as f:
                json.dump(d, f, indent=4)

        print('process complete')

        return d



from setuptools import setup

with open("README.md", "r") as fh:
	long_description = fh.read()


setup(
	name='NikeCA',
	version='0.1.48	',
	description='Standardize and Automate processes utilized by the DAMs at Nike in CA',
	py_modules=[
		"__init__",
		"NikeSF",
		"_SnowflakeData",
		"_BuildSearchQuery",
		"_IMPSummaryDashboard",
		"_SnowflakeDependencies",
		"_SnowflakePull",
		"NikeQA",
		"_QA",
		"_GitHub",
		"NikeCA"
	],
	package_dir={'': 'src'},
	classifiers=[
		"Programming Language :: Python :: 3",
		"Programming Language :: Python :: 3.11",
		"License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)",
		"Operating System :: OS Independent",
	],
	long_description=long_description,
	long_description_content_type="text/markdown",
	install_requires=[
		"wheel",
		"asn1crypto==1.5.1",
		"certifi==2022.12.7",
		"cffi==1.15.1",
		"charset-normalizer==2.1.1",
		"cryptography==39.0.1",
		"databricks==0.2",
		"databricks-sql==1.0.0",
		"databricks-sql-connector==2.2.1",
		"filelock==3.9.0",
		"gitdb==4.0.10",
		"github3.py==3.2.0",
		"GitPython==3.1.31",
		"greenlet==2.0.2",
		"httpx==0.23.3",
		"idna==3.4",
		"jupyter-contrib-core==0.4.2",
		"jupyter-contrib-nbextensions==0.7.0",
		"jupyter-events==0.6.3",
		"jupyter-highlight-selected-word==0.2.0",
		"jupyter-nbextensions-configurator==0.6.1",
		"jupyter-ydoc==0.2.2",
		"jupyter_client==8.0.3",
		"jupyter_core==5.2.0",
		"jupyter_server==2.3.0",
		"jupyter_server_fileid==0.8.0",
		"jupyter_server_terminals==0.4.4",
		"jupyter_server_ydoc==0.6.1",
		"jupyterlab==3.6.1",
		"jupyterlab-pygments==0.2.2",
		"jupyterlab_server==2.19.0",
		"lz4==4.3.2",
		"numpy==1.23.4",
		"oauthlib==3.2.2",
		"oscrypto==1.3.0",
		"pandas==1.5.3",
		"polars==0.16.17",
		"pyarrow==10.0.1",
		"pycparser==2.21",
		"pycryptodomex==3.17",
		"PyGithub==1.58.0",
		"PyJWT==2.6.0",
		"pyOpenSSL==23.0.0",
		"pyspark==3.3.2",
		"pystache==0.6.0",
		"python-dateutil==2.8.2",
		"pytz==2022.7.1",
		"requests==2.28.2",
		"six==1.16.0",
		"smmap==5.0.0",
		"snowflake-connector-python==3.0.0",
		"snowflake-sqlalchemy==1.4.6",
		"SQLAlchemy==1.4.46",
		"thrift==0.16.0",
		"typing_extensions==4.5.0",
		"urllib3==1.26.14",
		"xcrun==0.4",
		"configparser~=5.3.0",
	],
)
